//Andrew Osthoff, assignment 1a
#include <stdlib.h>
#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <math.h>
#include <float.h>
#include <time.h>
#include <vector>
#include <iostream>
#include <cassert>

#define MAX_PPM_DIM 8192
#define SHADOW_EPSILON .01
#define INTERSECT_EPSILON .0001
#define POINT_LIGHT_RADIUS .25
#define RAYS_PER_SHADOW_BUNDLE 32
#define SOFT_SHADOWS 0
#define MAX_PHONG_RECURSIONS 6

enum { NONE=0, SPHERE, POLYGON, ELLIPSOID};

#include "osl.h"
#include "linalg.h"
#include "renderer.cpp"
#include "parser.h"



int main(int argc, char* argv[])
{
  srand((unsigned int)time(NULL));
  renderState state;
  //vertex indexing should start at 1 per class instructions
  state.vertCount = 0;
  state.vertNormsCount = 0;
  state.textureCoordinateCount = 0;
  state.textureCoordinates.push_back({0,0});
  state.verts.push_back({0,0,0});
  state.vertNorms.push_back({0,0,0});
  state.textures.push_back({0,0,NULL});
  state.parallelMode = 0;

  if (argc < 2)
  {
    std::cerr << "Error: you must specify an input file, " << "e.g. \"./example ../examplefile.txt\"" << std::endl; 
    return 1;
  }
  //open input file
  FILE* inputFile;
  inputFile = fopen(argv[1], "r");
  if(inputFile == NULL)
  {
    std::cerr << "Error: could not open " << argv[1] << std::endl; 
    return 1;
  }

  //copy whole file to memory (returns null on failure)
  char* inputStr = wholeFileToMemory(inputFile);
  if(!inputStr)
  {
    std::cerr << "Error: could not read input file into memory." << std::endl;
    fclose(inputFile);
    return 1; 
  }

  //done with input file
  if(fclose(inputFile) == EOF)
  {
    std::cerr << "Error: this is bad. EOF was returned on input file's fclose, which means there may have been a system-level write error." << std::endl; 
    return 1;
  }

  if(!parseToRenderState(inputStr, &state))
  {
    return 1; //error parsing state, error messages handled in parseToRenderState()
  }
    free(inputStr);
  
  color* pixelBuffer = (color*) calloc(state.imsizeW*state.imsizeH, sizeof(color));
  if(!buildScene(state, pixelBuffer))
  {
    return 1; //error rendering scene, error messages handled in buildScene()
  }

  //gross bit of hasty code to write output file name
  //copies the input file argument character by character until a '.'' is reached, 
  //then appends ppm to the string
  int counter = strlen(argv[1]) +1;
  char* outputName = (char*) malloc(counter);
  if(outputName != NULL)
  {
    char* i = outputName;
    char* c = argv[1]; 
    *i = *c;
    while(*c != '.'&&*c!= '\0')
    {
      i++;
      c++;
      *i = *c;
      counter --;
    }
    if(*i == '.' && counter == 5)
    {
      *i++;
      *i++ = 'p';
      *i++ = 'p';
      *i++ = 'm';
      *i++ = '\0';
    }
    else
    {
      std::cerr << "Error: could not create output name. Are you sure you gave a .txt file for input?" << std::endl; 
	  return 1;
    }
  }

  else
  {
	  std::cerr << "Error: could not allocate memory for output name string." << std::endl; 
	  return 1;
  }

  //create the output file
  FILE* outputFile = fopen(outputName, "w");
  if(!outputPPM(state.imsizeW, state.imsizeH, pixelBuffer, outputFile))
  {
    return 1; //error creating output file, error messages handled in outputPPM()
  }

  fclose(outputFile);
  free(outputName);
  free(pixelBuffer);
  return 0;
}