struct point3f
{
  float x, y, z;
};

struct point2f
{
  float x, y;
};

struct vec3f
{
  float x, y, z;
};

struct triangle
{
  point3f v1, v2, v3;
};

struct polygon
{
  point3f p0, p1, p2;
  int materialIndex, textureIndex;
  vec3f n, e1, e2, shadingNorm;
  float d, area;
  int vert0Norm, vert1Norm, vert2Norm;
  int tex1, tex2, tex3;
};

struct sphere 
{
  point3f center;
  float radius;
  int materialIndex, textureIndex;
};

//OVERLOADED OPERATORS

vec3f operator+(vec3f a, vec3f b)
{
  return {a.x + b.x, a.y + b.y, a.z + b.z};
}

vec3f operator-(vec3f a, vec3f b)
{
  return {a.x - b.x, a.y - b.y, a.z - b.z};
}

vec3f operator-(vec3f a)
{
  return {-a.x, -a.y , -a.z};
}

// the sum or difference of 2 3d points is a vector: 
vec3f operator+(point3f a, point3f b)
{
  return {a.x + b.x, a.y + b.y, a.z + b.z};
}

vec3f operator-(point3f a, point3f b)
{
  return {a.x - b.x, a.y - b.y, a.z - b.z};
}

vec3f operator*(float c, vec3f v)
{
  return {c * v.x, c * v.y, c * v.z};
}

vec3f operator/(vec3f v, float c)
{
  return { v.x/c, v.y/c, v.z/c};
}

point3f operator+(point3f p, vec3f v)
{
  return {p.x + v.x, p.y + v.y, p.z + v.z};
}

point3f operator-(point3f p, vec3f v)
{
  return {p.x - v.x, p.y - v.y, p.z - v.z};
}

//BASIC LIN ALG

float innerProduct(vec3f v1, vec3f v2)
{
  return (v1.x*v2.x + v1.y*v2.y + v1.z*v2.z);
}

vec3f crossProduct(vec3f A, vec3f B)
{
  float x, y, z;
  x = (A.y*B.z - B.y*A.z);
  y = (B.x*A.z - A.x*B.z );
  z = (A.x*B.y - B.x*A.y);
  return  { x, y, z };
}


//DERIVED LIN ALG
vec3f normalizeVec3(vec3f v)
{
  float mag = sqrt(innerProduct(v,v));
  return {v.x/mag,v.y/mag,v.z/mag};
}

polygon vertsToPoly(point3f p0, point3f p1, point3f p2)
{
  vec3f e1 = p1-p0;
  vec3f e2 = p2 - p0;
  vec3f n = crossProduct(e1, e2);
  float area = .5*sqrt(innerProduct(n,n));
  float d = -(n.x*p0.x + n.y*p0.y +n.z*p0.z);
  vec3f shadingNorm = normalizeVec3(n);


  // vertecies; materialIndex; textureIndex; areaNorm and stored vectors; 
  // shadingNorm (for lighting); d term in barycentric coordinates; area; 
  // vertexNorms (initialize to out of range), indecies to texture coordinates (initialize to out of range)
  return{ p0, p1, p2, 0, 0, n, e1, e2, shadingNorm, d, area, -1, -1, -1, -1,-1,-1 };
}
