//Defines render state and objects that appear in the scene
struct material
{
 float r, g, b, rSpec, gSpec, bSpec, ambRef, difRef, specRef;
 int n, textureIndex;
 float opacity, refIndex;
};

struct color
{
  float r, g, b;
};

struct texture 
{
  int width, height; 
  float* data;
};

struct pointLight
{
  point3f center;
  color c;
};

struct directionalLight
{
  vec3f direction;
  color c;
};

struct ellipse
{
	point3f center;
	float xRad, yRad, zRad;
	int materialIndex;
};

struct renderState
{
  int vertCount, vertNormsCount, textureCoordinateCount;
  int imsizeW, imsizeH;
  int parallelMode;
  float fovv;
  vec3f viewdir;
  vec3f updir;
  vec3f deltaXPlaneToPix, deltaYPlaneToPix; 
  point3f eye;
  color bkgcolor;
  std::vector <sphere> spheres;
  std::vector <point3f> verts;
  std::vector <vec3f> vertNorms;
  std::vector <polygon> polys;
  std::vector <ellipse> ellipses;
  std::vector <material> materials;
  std::vector <texture> textures;
  std::vector <point2f> textureCoordinates;
  std::vector <pointLight> pointLights;
  std::vector <directionalLight> directionalLights;
};


color operator*(float i, color c)
{
  return {i*c.r, i*c.g,i*c.b};
}

color operator+(color c1, color c2)
{
  return {c1.r + c2.r, c1.g + c2.g, c1.b + c2.b};
}

color phongLighting(vec3f* viewRay, point3f* intersectPoint, vec3f* surfaceNorm, material* mat, renderState* state, int traceNumber, float refractionIndexes[MAX_PHONG_RECURSIONS+1]);
color closestShapeAndLighting(renderState* state, point3f origin, vec3f viewRay, int traceNumber, float refractionIndexes[MAX_PHONG_RECURSIONS+1] );